package springbatchdemo;

import org.springframework.batch.core.*;
import org.springframework.batch.core.configuration.annotation.*;
import org.springframework.batch.core.launch.support.RunIdIncrementer;
import org.springframework.batch.core.scope.context.ChunkContext;
import org.springframework.batch.core.step.tasklet.Tasklet;
import org.springframework.batch.item.ExecutionContext;
import org.springframework.batch.repeat.RepeatStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.*;

/**
 * JobConfiguration fuer den JUnit-Modultest "ConcurrentJobTest", der zwei Jobs parallel startet,
 * und testet, dass die parallel ausgefuehrten Steps sich nicht gegenseitig beeinflussen.
 */
@Configuration
@EnableBatchProcessing
public class ConcurrentJobConfiguration
{
   public static final String JOB_NAME      = "meinConcurrentJob";
   public static final String MEIN_PARM_KEY = "MeinParm";
   public static final String MEIN_MSG_KEY  = "Msg";
   @Autowired private JobBuilderFactory  jobBuilderFactory;
   @Autowired private StepBuilderFactory stepBuilderFactory;

   @Autowired MeinContextHolder ctxHolder;

   public interface MeinContextHolder
   {
      String getText();
      void setText( String text );
   }

   public class MeinContextHolderImpl implements MeinContextHolder
   {
      private String text = "";
      @Override public String getText() { return text; }
      @Override public void setText( String text ) { this.text = text; }
   }

   /** Context-Holder-Bean mit Job-Scope */
   @Bean
   @Scope( value="job", proxyMode= ScopedProxyMode.INTERFACES )
   public MeinContextHolder createMeinContextHolder()
   {
      return new MeinContextHolderImpl();
   }

   /** Wenn der Job mehrfach gestartet wird, wird dieser Step multi-threaded parallel durchlaufen */
   @Bean
   public Step meinConcurrentStep()
   {
      return stepBuilderFactory.get( "meinConcurrentStep" ).tasklet( new Tasklet()
      {
         @Override
         public RepeatStatus execute( StepContribution sc, ChunkContext cc )
         {
            String meinParm = cc.getStepContext().getStepExecution().getJobParameters().getString( MEIN_PARM_KEY );
            ctxHolder.setText( ctxHolder.getText() + meinParm );
            cc.getStepContext().getStepExecution().getJobExecution().getExecutionContext().put( MEIN_MSG_KEY, ctxHolder.getText() );
            return RepeatStatus.FINISHED;
         }
      } ).build();
   }

   @Bean
   public Job meinConcurrentJob() throws Exception
   {
      return jobBuilderFactory.get( JOB_NAME ).incrementer( new RunIdIncrementer() )
              .start( meinConcurrentStep() )
              .next(  meinConcurrentStep() )
              .next(  meinConcurrentStep() )
              .build();
   }
}
