package springbatchdemo;

import static org.hamcrest.Matchers.is;
import static org.junit.Assert.*;
import org.junit.*;
import org.junit.runner.RunWith;
import org.springframework.batch.core.*;
import org.springframework.batch.core.launch.JobLauncher;
import org.springframework.beans.factory.annotation.*;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

@ContextConfiguration( classes = MeineApplication.class )
@RunWith( SpringJUnit4ClassRunner.class )
public class ConditionalFlowJobTest
{
   @Autowired private JobLauncher jobLauncher;
   @Autowired @Qualifier( "meinConditionalFlowJobAbc" ) private Job jobAbc;
   @Autowired @Qualifier( "meinConditionalFlowJobXyz" ) private Job jobXyz;

   @Test
   public void testJobOk() throws Exception
   {
      System.out.println( "\nJoblauf mit Job-Parameter 'ok':" );
      JobExecution je = jobLauncher.run( jobAbc, new JobParametersBuilder().addString(
            ConditionalFlowJobConfiguration.OK_ODER_FEHLER, "ok" ).toJobParameters() );
      // Ueberpruefe Job-Ergebnis:
      String[] steps = new String[] { "arbeitsStep", "okStep", "abschliessenderStep" };
      Assert.assertEquals( "jobAbc", je.getJobInstance().getJobName() );
      Assert.assertEquals( BatchStatus.COMPLETED, je.getStatus() );
      Assert.assertEquals( 0, je.getAllFailureExceptions().size() );
      Assert.assertEquals( 3, je.getStepExecutions().size() );
      int i = 0;
      for( StepExecution se : je.getStepExecutions() ) {
         System.out.println( "StepExecution " + se.getId() + ": CommitCount = " + se.getCommitCount() +
                             ", Status = " + se.getStatus() + ", ExitStatus = " + se.getExitStatus().getExitCode() +
                             ", StepName = " + se.getStepName() );
         Assert.assertEquals( BatchStatus.COMPLETED, se.getStatus() );
         Assert.assertEquals( 1, se.getCommitCount() );
         Assert.assertEquals( steps[i++], se.getStepName() );
      }
   }

   @Test
   public void testJobMitFehler() throws Exception
   {
      System.out.println( "\nJoblauf mit Job-Parameter 'Fehler':" );
      JobExecution je = jobLauncher.run( jobAbc, new JobParametersBuilder().addString(
            ConditionalFlowJobConfiguration.OK_ODER_FEHLER, "Fehler" ).toJobParameters() );
      // Ueberpruefe Job-Ergebnis:
      String[]      steps   = new String[]      { "arbeitsStep",       "fehlerbehandlungsStep", "abschliessenderStep" };
      BatchStatus[] baStati = new BatchStatus[] { BatchStatus.ABANDONED, BatchStatus.COMPLETED, BatchStatus.COMPLETED };
      ExitStatus[]  exStati = new ExitStatus[]  { ExitStatus.FAILED,      ExitStatus.COMPLETED,  ExitStatus.COMPLETED };
      int[]         commits = new int[] { 0, 1, 1 };
      Assert.assertEquals( "jobAbc", je.getJobInstance().getJobName() );
      Assert.assertEquals( BatchStatus.COMPLETED, je.getStatus() );
      Assert.assertEquals( 1, je.getAllFailureExceptions().size() );
      Assert.assertEquals( 3, je.getStepExecutions().size() );
      int i = 0;
      for( StepExecution se : je.getStepExecutions() ) {
         System.out.println( "StepExecution " + se.getId() + ": CommitCount = " + se.getCommitCount() +
                             ", Status = " + se.getStatus() + ", ExitStatus = " + se.getExitStatus().getExitCode() +
                             ", StepName = " + se.getStepName() );
         Assert.assertEquals( commits[i],               se.getCommitCount() );
         Assert.assertEquals( baStati[i],               se.getStatus() );
         Assert.assertEquals( exStati[i].getExitCode(), se.getExitStatus().getExitCode() );
         Assert.assertEquals( steps[i++],               se.getStepName() );
      }
   }

   @Test
   public void testJobXyz() throws JobExecutionException
   {
      // Job Xyz im OK-Fall:
      JobExecution je1 = jobLauncher.run( jobXyz, new JobParametersBuilder().addString(
            ConditionalFlowJobConfiguration.OK_ODER_FEHLER, "ok" ).toJobParameters() );
      assertThat( "Job-Name falsch",                je1.getJobInstance().getJobName(), is( "jobXyz" ) );
      assertThat( "Job-Step-Anzahl falsch",         je1.getStepExecutions().size(), is( 4 ) );
      assertThat( "FailureExceptions",              je1.getAllFailureExceptions().size(), is( 0 ) );
      assertThat( "Job ist nicht COMPLETED",        je1.getStatus(), is( BatchStatus.COMPLETED ) );
      assertThat( "ExitStatus ist nicht COMPLETED", je1.getExitStatus(), is( ExitStatus.COMPLETED ) );
      for( StepExecution se : je1.getStepExecutions() ) {
         System.out.println( "StepExecution " + se.getId() + ": CommitCount = " + se.getCommitCount() +
                             ", Status = " + se.getStatus() + ", ExitStatus = " + se.getExitStatus().getExitCode() +
                             ", StepName = " + se.getStepName() );
      }
      // Job Xyz im Fehler-Fall:
      JobExecution je2 = jobLauncher.run( jobXyz, new JobParametersBuilder().addString(
            ConditionalFlowJobConfiguration.OK_ODER_FEHLER, "Fehler" ).toJobParameters() );
      assertThat( "Job-Name falsch",                je2.getJobInstance().getJobName(), is( "jobXyz" ) );
      assertThat( "Job-Step-Anzahl falsch",         je2.getStepExecutions().size(), is( 3 ) );
      assertThat( "FailureExceptions",              je2.getAllFailureExceptions().size(), is( 1 ) );
      assertThat( "Job ist nicht COMPLETED",        je2.getStatus(), is( BatchStatus.COMPLETED ) );
      assertThat( "ExitStatus ist nicht COMPLETED", je2.getExitStatus(), is( ExitStatus.COMPLETED ) );
      for( StepExecution se : je2.getStepExecutions() ) {
         System.out.println( "StepExecution " + se.getId() + ": CommitCount = " + se.getCommitCount() +
                             ", Status = " + se.getStatus() + ", ExitStatus = " + se.getExitStatus().getExitCode() +
                             ", StepName = " + se.getStepName() );
      }
   }
}
