package springbatchdemo;

import static org.hamcrest.Matchers.is;
import static org.junit.Assert.*;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.batch.core.*;
import org.springframework.batch.core.explore.JobExplorer;
import org.springframework.batch.core.launch.*;
import org.springframework.batch.core.repository.JobInstanceAlreadyCompleteException;
import org.springframework.beans.factory.annotation.*;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

@RunWith( SpringJUnit4ClassRunner.class )
@ContextConfiguration( classes = MeineApplication.class )
public class JobOperatorTest
{
   @Autowired private JobExplorer jobExplorer;
   @Autowired private JobOperator jobOperator;
   @Autowired private JobLauncher jobLauncher;
   @Autowired @Qualifier( "meinConditionalFlowJobAbc" ) private Job jobAbc;

   /** Mehrfache Job-Ausfuehrung mit JobLauncher --> Fehler */
   @Test
   public void testMehrfacheJobAusfuehrungMitJobLauncherMitFehler() throws JobExecutionException
   {
      JobParameters jp = new JobParametersBuilder()
            .addString( "JobLauncher-Test", "nur-einmal-ausfuehrbar" )
            .addString( "OK_ODER_FEHLER", "ok" ).toJobParameters();

      // Erste Job-Ausfuehrung:
      JobExecution je = jobLauncher.run( jobAbc, jp );
      assertThat( "Job-Name falsch",         je.getJobInstance().getJobName(), is( "jobAbc" ) );
      assertThat( "Job-Step-Anzahl falsch",  je.getStepExecutions().size(), is( 3 ) );
      assertThat( "Job ist nicht COMPLETED", je.getStatus(), is( BatchStatus.COMPLETED ) );

      // Zweite Job-Ausfuehrung mit denselben JobParametern fuehrt zu Fehler, weil der im Job definierte JobParametersIncrementer ignoriert wird:
      try {
         jobLauncher.run( jobAbc, jp );
         fail( "'JobInstanceAlreadyCompleteException' muss geworfen werden" );
      } catch( JobInstanceAlreadyCompleteException ex ) {
         /* ok */
      }
   }

   /** Mehrfache Job-Ausfuehrung mit JobOperator --> funktioniert */
   @Test
   public void testMehrfacheJobAusfuehrungMitJobOperatorOhneFehler() throws JobExecutionException
   {
      System.out.println( "\n---- Vorhandene Jobs: " + jobOperator.getJobNames() );

      // Erste Job-Ausfuehrung:
      Long executionId1 = jobOperator.start( "jobAbc", "JobOperator-Test=mehrfach-ausfuehrbar,OK_ODER_FEHLER=ok" );

      // Zweite Job-Ausfuehrung funktioniert, weil der im Job definierte JobParametersIncrementer verwendet wird:
      Long executionId2 = jobOperator.startNextInstance( "jobAbc" );
      JobExecution je   = jobExplorer.getJobExecution( executionId2 );
      assertThat( "Job-Name falsch",         je.getJobInstance().getJobName(), is( "jobAbc" ) );
      assertThat( "Job-Step-Anzahl falsch",  je.getStepExecutions().size(), is( 3 ) );
      assertThat( "Job ist nicht COMPLETED", je.getStatus(), is( BatchStatus.COMPLETED ) );
   }
}
