package springbatchdemo;

import org.junit.*;
import org.junit.runner.RunWith;
import org.springframework.batch.core.*;
import org.springframework.batch.core.launch.JobLauncher;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

@ContextConfiguration( classes = MeineApplication.class )
@RunWith( SpringJUnit4ClassRunner.class )
public class TaskletJobTest
{
   @Autowired private JobLauncher jobLauncher;
   @Autowired private Job job;

   @Test
   public void testJobOk() throws Exception
   {
      final int anzahlSteps = 7;
      boolean b = false;
      System.out.println( "\nJoblauf mit Job-Parameter anzahlSteps=" + anzahlSteps + ":" );
      JobExecution je = jobLauncher.run( job, new JobParametersBuilder().addString(
            TaskletJobConfiguration.ANZAHLSTEPS_KEY, "" + anzahlSteps ).toJobParameters() );
      // Ueberpruefe Job-Ergebnis:
      Assert.assertEquals( BatchStatus.COMPLETED, je.getStatus() );
      Assert.assertEquals( 0, je.getAllFailureExceptions().size() );
      Assert.assertEquals( 3, je.getStepExecutions().size() );
      for( StepExecution se : je.getStepExecutions() ) {
         System.out.println( "StepExecution " + se.getId() + ": StepName = " + se.getStepName() +
                             ", CommitCount = " + se.getCommitCount() );
         Assert.assertEquals( ( b = !b ) ? 1 : anzahlSteps, se.getCommitCount() );
         Assert.assertEquals( BatchStatus.COMPLETED, se.getStatus() );
      }
   }

   @Test
   public void testJobMitFehler() throws Exception
   {
      System.out.println( "\nJoblauf mit fehlerhaftem Job-Parameter (Text statt Zahl):" );
      JobExecution je = jobLauncher.run( job, new JobParametersBuilder().addString(
            TaskletJobConfiguration.ANZAHLSTEPS_KEY, "xx" ).toJobParameters() );
      // Ueberpruefe Job-Ergebnis:
      Assert.assertEquals( BatchStatus.FAILED, je.getStatus() );
      Assert.assertEquals( 2, je.getStepExecutions().size() );
      for( StepExecution se : je.getStepExecutions() ) {
         System.out.println( "StepExecution " + se.getId() + ": StepName = " + se.getStepName() +
                             ", CommitCount = " + se.getCommitCount() +
                             ", BatchStatus = " + se.getStatus() + ", ExitStatus = " + se.getExitStatus().getExitCode() );
      }
      Assert.assertEquals( 1, je.getAllFailureExceptions().size() );
      for( Throwable ex : je.getAllFailureExceptions() ) {
         System.out.println( ex );
      }
   }
}
